// Copyright 2014, Additive Regularization of Topic Models.

#include "artm/score_sandbox/theta_snippet.h"

#include "artm/core/exceptions.h"
#include "artm/core/topic_model.h"
#include "artm/core/protobuf_helpers.h"

namespace artm {
namespace score_sandbox {

void ThetaSnippet::AppendScore(
    const Item& item,
    const std::vector<artm::core::Token>& token_dict,
    const artm::core::TopicModel& topic_model,
    const artm::ModelConfig& model_config,
    const std::vector<float>& theta,
    Score* score) {
  int topics_size = topic_model.topic_size();

  ThetaSnippetScore theta_snippet_score;
  theta_snippet_score.add_item_id(item.id());
  auto theta_snippet_item = theta_snippet_score.add_values();
  for (int topic_index = 0; topic_index < topics_size; ++topic_index) {
    theta_snippet_item->add_value(theta[topic_index]);
  }

  AppendScore(theta_snippet_score, score);
}

std::string ThetaSnippet::stream_name() const {
  return config_.stream_name();
}

std::shared_ptr<Score> ThetaSnippet::CreateScore() {
  return std::make_shared<ThetaSnippetScore>();
}

void ThetaSnippet::AppendScore(const Score& score, Score* target) {
  std::string error_message = "Unable downcast Score to SparsityThetaScore";
  const ThetaSnippetScore* theta_snippet_score = dynamic_cast<const ThetaSnippetScore*>(&score);
  if (theta_snippet_score == nullptr) {
    BOOST_THROW_EXCEPTION(::artm::core::InternalError(error_message));
  }

  ThetaSnippetScore* theta_snippet_target = dynamic_cast<ThetaSnippetScore*>(target);
  if (theta_snippet_target == nullptr) {
    BOOST_THROW_EXCEPTION(::artm::core::InternalError(error_message));
  }

  if (config_.item_count() <= 0 || theta_snippet_score->values_size() == 0)
    return;

  while (theta_snippet_target->values_size() < config_.item_count()) {
    theta_snippet_target->add_item_id(-1);
    artm::FloatArray* values_target = theta_snippet_target->add_values();
    for (int i = 0; i < theta_snippet_score->values(0).value_size(); ++i)
      values_target->add_value(0.0f);
  }

  for (int item_index = 0; item_index < theta_snippet_score->item_id_size(); item_index++) {
    int item_id = theta_snippet_score->item_id(item_index);
    if (item_id < 0)
      continue;

    theta_snippet_target->set_item_id(item_id % config_.item_count(), item_id);
    artm::FloatArray* values_target = theta_snippet_target->mutable_values(item_id % config_.item_count());
    values_target->Clear();

    auto theta_snippet_item_score = theta_snippet_score->values(item_index);
    for (int topic_index = 0; topic_index < theta_snippet_item_score.value_size(); topic_index++) {
      values_target->add_value(theta_snippet_item_score.value(topic_index));
    }
  }
}

}  // namespace score_sandbox
}  // namespace artm
